window.Gleam.initSearch([{"doc":"ormlette","title":"ormlette","content":"# ormlette\n\nOrmlette is a delightful ORM (?) for Gleam. It&#39;s features include but are not limited to:\n\n- A (pretty awesome) schema DSL\n- A powerful query DSL that can be exported to `gleam-cake` Select statements, and used directly with `cake`\n- A CLI to automatically generate easy-use types\n\n&gt; Ormlette is currently in the very early stages of development, so advice/feature requests/bug reports are super helpful!\n\n&gt; [!NOTE]\n&gt; THESE DOCS ARE NOT COMPLETE YET!\n\n## Schema DSL\n\nThe schema DSL is the main way to define your database schema within ormlette. It allows you to define tables, columns, and relationships between tables.\n\nHere is what it looks like:\n\n```gleam\nimport ormlette/schema/create as c\n\npub fn users_table() {\n  c.define_table(&quot;users&quot;, [\n    c.serial(&quot;id&quot;) |&gt; c.primary,\n    c.text(&quot;name&quot;) |&gt; c.nullable,\n  ])\n}\n```\n\nTo generate everything you need to work with databases, run `gleam run -m ormlette` and look in the `src/eggs` directory.\n\n## Query DSL\n\nThe query DSL is the main way to build queries with ormlette. It allows you to select, insert, update, and delete data from your database. It is **heavily** based on `gleam-cake`, and uses it under the hood. Luckily, if you need to use features that `cake` supports and ormlette doesn&#39;t, you can just `q.export` the query, and use it directly with `cake`!\n\nHere is what the query DSL looks like:\n\n```gleam\nq.from_table(posts_table)\n    |&gt; q.inner_join(users_table)\n    |&gt; q.select([&quot;users.name&quot;, &quot;posts.id&quot;, &quot;posts.user_id&quot;])\n    |&gt; q.sql\n    |&gt; io.debug()\n    |&gt; run.run(db, users_posts_decoder.continuation) // this decoder is generated for us!\n    |&gt; io.debug\n```\n\nHere is the magical part: the decoder puts the results into a type perfectly, every time. No matter what you select, how you select it, the order, etc., it is all put into the correct type, every time. All with one command!\n\n## CLI\n\nThe CLI is how you generate our &quot;universal decoders,&quot; decoder types, and usage types. So what are all of these?\n\n### Universal Decoders\n\nUniversal decoders are just dynamic decoders. The special part, though, is that they allow you to decode queries EVERY time. This means that even if you make two different `select` queries, each with a different order of selected columns, the **same** decoder will work with both queries!\n\nMoreover, it will also automatically generate join types and decoders with NO extra work from you. This means that if you have two tables that are related, there is no issue decoding these dynamic values! Pretty epic.\n\n### Decoder Types\n\nDecoder types are what the dynamic decoders decode the dynamic values into. This means that you will be able to reference columns of a row, without needing any extra work, and with them being the type you want!\n\n### Usage Types\n\nUsage types are not strictly necessary to enjoy using `ormlette`. But, (in my opinion) make everything much more pleasent. They gaurentee that you do not\na) miss-spell a column,\nb) reference a column that doesn&#39;t exist, and\nc) provide code completeion for the different columns!\n\nUsage types allow you to do something like:\n\n```gleam\nimport eggs/tables as t\n\nt.users().id // this gets the users table id column and turns it into something that can be used in a seelct column, for instance\n```\n\nThey are automatically generated, and in my opinion are worth using!\n","url":"index.html"},{"doc":"ormlette/cli/schema","title":"find_schema_files","content":"pub fn find_schema_files() -&gt; Option(List(String))\n Main function. Just finds the things, see module description.\nSynonyms:\normlette/cli/schema.find_schema_files\normlette/cli/schema find_schema_files","url":"ormlette/cli/schema.html#find_schema_files"},{"doc":"ormlette/cli/schema","title":"ormlette/cli/schema","content":" This module is responsible for finding all the schema files in the project.\n It uses the `globlin` and `globlin_fs` libraries to find all the files in the `src/schema` directory.\n It then strips the file extension and returns the list of schema names.","url":"ormlette/cli/schema.html"},{"doc":"ormlette/templates/generate","title":"render","content":"pub fn render(paths paths: List(String)) -&gt; String\n\nSynonyms:\normlette/templates/generate.render\normlette/templates/generate render","url":"ormlette/templates/generate.html#render"},{"doc":"ormlette/templates/generate","title":"render_builder","content":"pub fn render_builder(paths paths: List(String)) -&gt; StringBuilder\n\nSynonyms:\normlette/templates/generate.render_builder\normlette/templates/generate render_builder","url":"ormlette/templates/generate.html#render_builder"},{"doc":"ormlette/templates/generate","title":"ormlette/templates/generate","content":"","url":"ormlette/templates/generate.html"},{"doc":"ormlette/schema/create","title":"Column","content":"pub type Column {\n  Column(\n    name: String,\n    type_: ColumnType,\n    is_primary: Bool,\n    is_nullable: Bool,\n    is_unique: Bool,\n    default: Option(dynamic.Dynamic),\n    references: Option(Reference),\n  )\n}\n\nColumn(\n  name: String,\n  type_: ColumnType,\n  is_primary: Bool,\n  is_nullable: Bool,\n  is_unique: Bool,\n  default: Option(dynamic.Dynamic),\n  references: Option(Reference),\n)\n\n\nSynonyms:\normlette/schema/create.Column\normlette/schema/create Column","url":"ormlette/schema/create.html#Column"},{"doc":"ormlette/schema/create","title":"ColumnType","content":"pub type ColumnType {\n  Int\n  String\n  Bool\n  ForeignKey\n  Serial\n}\n\nInt\n\n\nString\n\n\nBool\n\n\nForeignKey\n\n\nSerial\n\n\nSynonyms:\normlette/schema/create.ColumnType\normlette/schema/create ColumnType","url":"ormlette/schema/create.html#ColumnType"},{"doc":"ormlette/schema/create","title":"DbType","content":"pub type DbType {\n  Integer\n  Text\n  Real\n  Blob\n}\n\nInteger\n\n\nText\n\n\nReal\n\n\nBlob\n\n\nSynonyms:\normlette/schema/create.DbType\normlette/schema/create DbType","url":"ormlette/schema/create.html#DbType"},{"doc":"ormlette/schema/create","title":"Reference","content":"pub type Reference {\n  Reference(\n    table: Table,\n    column: String,\n    on_delete: Option(String),\n    on_update: Option(String),\n  )\n}\n\nReference(\n  table: Table,\n  column: String,\n  on_delete: Option(String),\n  on_update: Option(String),\n)\n\n\nSynonyms:\normlette/schema/create.Reference\normlette/schema/create Reference","url":"ormlette/schema/create.html#Reference"},{"doc":"ormlette/schema/create","title":"Table","content":"pub type Table {\n  Table(name: String, columns: List(Column))\n}\n\nTable(name: String, columns: List(Column))\n\n\nSynonyms:\normlette/schema/create.Table\normlette/schema/create Table","url":"ormlette/schema/create.html#Table"},{"doc":"ormlette/schema/create","title":"dbtype_to_string","content":"pub fn dbtype_to_string(dbtype: ColumnType) -&gt; String\n\nSynonyms:\normlette/schema/create.dbtype_to_string\normlette/schema/create dbtype_to_string","url":"ormlette/schema/create.html#dbtype_to_string"},{"doc":"ormlette/schema/create","title":"default","content":"pub fn default(col: Column, value: a) -&gt; Column\n\nSynonyms:\normlette/schema/create.default\normlette/schema/create default","url":"ormlette/schema/create.html#default"},{"doc":"ormlette/schema/create","title":"define_table","content":"pub fn define_table(name: String, columns: List(Column)) -&gt; Table\n\nSynonyms:\normlette/schema/create.define_table\normlette/schema/create define_table","url":"ormlette/schema/create.html#define_table"},{"doc":"ormlette/schema/create","title":"foreign_key","content":"pub fn foreign_key(\n  col: Column,\n  references_table: Table,\n  references_column: String,\n) -&gt; Column\n\nSynonyms:\normlette/schema/create.foreign_key\normlette/schema/create foreign_key","url":"ormlette/schema/create.html#foreign_key"},{"doc":"ormlette/schema/create","title":"format_column_sql","content":"pub fn format_column_sql(column: Column) -&gt; String\n\nSynonyms:\normlette/schema/create.format_column_sql\normlette/schema/create format_column_sql","url":"ormlette/schema/create.html#format_column_sql"},{"doc":"ormlette/schema/create","title":"int","content":"pub fn int(name: String) -&gt; Column\n\nSynonyms:\normlette/schema/create.int\normlette/schema/create int","url":"ormlette/schema/create.html#int"},{"doc":"ormlette/schema/create","title":"nullable","content":"pub fn nullable(col: Column) -&gt; Column\n\nSynonyms:\normlette/schema/create.nullable\normlette/schema/create nullable","url":"ormlette/schema/create.html#nullable"},{"doc":"ormlette/schema/create","title":"primary","content":"pub fn primary(col: Column) -&gt; Column\n\nSynonyms:\normlette/schema/create.primary\normlette/schema/create primary","url":"ormlette/schema/create.html#primary"},{"doc":"ormlette/schema/create","title":"serial","content":"pub fn serial(name: String) -&gt; Column\n\nSynonyms:\normlette/schema/create.serial\normlette/schema/create serial","url":"ormlette/schema/create.html#serial"},{"doc":"ormlette/schema/create","title":"text","content":"pub fn text(name: String) -&gt; Column\n\nSynonyms:\normlette/schema/create.text\normlette/schema/create text","url":"ormlette/schema/create.html#text"},{"doc":"ormlette/schema/create","title":"unique","content":"pub fn unique(col: Column) -&gt; Column\n\nSynonyms:\normlette/schema/create.unique\normlette/schema/create unique","url":"ormlette/schema/create.html#unique"},{"doc":"ormlette/schema/create","title":"ormlette/schema/create","content":"","url":"ormlette/schema/create.html"},{"doc":"ormlette/templates/utils/to_string","title":"Style","content":"pub type Style {\n  Append\n  Write\n}\n\nAppend\n\n\nWrite\n\n\nSynonyms:\normlette/templates/utils/to_string.Style\normlette/templates/utils/to_string Style","url":"ormlette/templates/utils/to_string.html#Style"},{"doc":"ormlette/templates/utils/to_string","title":"coltype","content":"pub fn coltype(column: Column) -&gt; String\n\nSynonyms:\normlette/templates/utils/to_string.coltype\normlette/templates/utils/to_string coltype","url":"ormlette/templates/utils/to_string.html#coltype"},{"doc":"ormlette/templates/utils/to_string","title":"decode_type","content":"pub fn decode_type(column: Column) -&gt; String\n\nSynonyms:\normlette/templates/utils/to_string.decode_type\normlette/templates/utils/to_string decode_type","url":"ormlette/templates/utils/to_string.html#decode_type"},{"doc":"ormlette/templates/utils/to_string","title":"to_file","content":"pub fn to_file(\n  info: String,\n  path path: String,\n  style style: Style,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\normlette/templates/utils/to_string.to_file\normlette/templates/utils/to_string to_file","url":"ormlette/templates/utils/to_string.html#to_file"},{"doc":"ormlette/templates/utils/to_string","title":"ormlette/templates/utils/to_string","content":"","url":"ormlette/templates/utils/to_string.html"},{"doc":"ormlette/cli/generate","title":"create_generate_file","content":"pub fn create_generate_file() -&gt; Result(Nil, FileError)\n\nSynonyms:\normlette/cli/generate.create_generate_file\normlette/cli/generate create_generate_file","url":"ormlette/cli/generate.html#create_generate_file"},{"doc":"ormlette/cli/generate","title":"ormlette/cli/generate","content":"/ This uses a matcha template to generate a file that will be used to generate the schema files.\n","url":"ormlette/cli/generate.html"},{"doc":"ormlette","title":"gleam_run","content":"pub fn gleam_run() -&gt; Result(Nil, Nil)\n This runs the generated file with gleam to actually create the necesary files\n\nSynonyms:\normlette.gleam_run\normlette gleam_run","url":"ormlette.html#gleam_run"},{"doc":"ormlette","title":"main","content":"pub fn main() -&gt; Result(Nil, Nil)\n Creates the `src/eggs` directory and adds the files. Pretty cool! :)\n\nSynonyms:\normlette.main\normlette main","url":"ormlette.html#main"},{"doc":"ormlette","title":"ormlette","content":" This is the CLI tool&#39;s entry point\n TO use it, just run `gleam run -m ormlette` in your project, with the schema in the `src/schema` folder, named with the following rules:\n * File name = table name\n * A function named {table_name}_table that returns a table object\n","url":"ormlette.html"},{"doc":"ormlette/ir/ir","title":"ColumnConstraint","content":"pub type ColumnConstraint {\n  PrimaryKey\n  Nullable\n  Unique\n  ForeignKey(\n    references_table: TableIR,\n    references_column: String,\n    on_delete: Option(String),\n    on_update: Option(String),\n  )\n}\n Ir type for a column constraint, such as a `PRIMARY KEY`, `FOREIGN KEY`, `UNIQUE`, etc.\nPrimaryKey\n\n\nNullable\n\n\nUnique\n\n\nForeignKey(\n  references_table: TableIR,\n  references_column: String,\n  on_delete: Option(String),\n  on_update: Option(String),\n)\n\n\nSynonyms:\normlette/ir/ir.ColumnConstraint\normlette/ir/ir ColumnConstraint","url":"ormlette/ir/ir.html#ColumnConstraint"},{"doc":"ormlette/ir/ir","title":"ColumnIR","content":"pub type ColumnIR {\n  ColumnIR(\n    name: String,\n    type_: create.ColumnType,\n    constraints: List(ColumnConstraint),\n    default: Option(dynamic.Dynamic),\n  )\n}\n This is the IR type for a column.\nColumnIR(\n  name: String,\n  type_: create.ColumnType,\n  constraints: List(ColumnConstraint),\n  default: Option(dynamic.Dynamic),\n)\n\n\nSynonyms:\normlette/ir/ir.ColumnIR\normlette/ir/ir ColumnIR","url":"ormlette/ir/ir.html#ColumnIR"},{"doc":"ormlette/ir/ir","title":"TableIR","content":"pub type TableIR {\n  TableIR(name: String, columns: List(ColumnIR))\n}\n The IR type for a table\n It contains the table name and a list of columns (Basically the same as the `create.Table` type)\nTableIR(name: String, columns: List(ColumnIR))\n\n\nSynonyms:\normlette/ir/ir.TableIR\normlette/ir/ir TableIR","url":"ormlette/ir/ir.html#TableIR"},{"doc":"ormlette/ir/ir","title":"to_ir","content":"pub fn to_ir(table: Table) -&gt; TableIR\n This function converts a `create.Table` to a `TableIR`\nSynonyms:\normlette/ir/ir.to_ir\normlette/ir/ir to_ir","url":"ormlette/ir/ir.html#to_ir"},{"doc":"ormlette/ir/ir","title":"ormlette/ir/ir","content":" This is an `intermediate representation` (IR) for the schema definition to convert it to a SQL `CREATE TABLE` statement.\n The IR is a data structure that represents the schema in a way that is easy to convert to SQL.","url":"ormlette/ir/ir.html"},{"doc":"ormlette/ir/sql","title":"helper","content":"pub fn helper(column: ColumnIR) -&gt; String\n This needs a better name. But, it basically helps determine the SQL type of a column.\nSynonyms:\normlette/ir/sql.helper\normlette/ir/sql helper","url":"ormlette/ir/sql.html#helper"},{"doc":"ormlette/ir/sql","title":"to_sql","content":"pub fn to_sql(table_ir: TableIR) -&gt; String\n This converts IR to SQL\nSynonyms:\normlette/ir/sql.to_sql\normlette/ir/sql to_sql","url":"ormlette/ir/sql.html#to_sql"},{"doc":"ormlette/ir/sql","title":"ormlette/ir/sql","content":" This module helps convert the IR defined in `ir/ir` to SQL strings.","url":"ormlette/ir/sql.html"},{"doc":"ormlette/query/helpers","title":"select_to_sql","content":"pub fn select_to_sql(select: Select) -&gt; #(String, List(Param))\n This turns a select value to a prepared statement. Literally just calling cake functions.\nSynonyms:\normlette/query/helpers.select_to_sql\normlette/query/helpers select_to_sql","url":"ormlette/query/helpers.html#select_to_sql"},{"doc":"ormlette/query/helpers","title":"ormlette/query/helpers","content":"  These functions are helpers for the query building. Are NOT used much, might remove later","url":"ormlette/query/helpers.html"},{"doc":"ormlette/query/query","title":"Query","content":"pub type Query {\n  Query(table: create.Table, select: select.Select)\n}\n\nQuery(table: create.Table, select: select.Select)\n\n\nSynonyms:\normlette/query/query.Query\normlette/query/query Query","url":"ormlette/query/query.html#Query"},{"doc":"ormlette/query/query","title":"equals","content":"pub fn equals(query: Query, column: String, value: a) -&gt; Query\n\nSynonyms:\normlette/query/query.equals\normlette/query/query equals","url":"ormlette/query/query.html#equals"},{"doc":"ormlette/query/query","title":"export","content":"pub fn export(query: Query) -&gt; Select\n\nSynonyms:\normlette/query/query.export\normlette/query/query export","url":"ormlette/query/query.html#export"},{"doc":"ormlette/query/query","title":"from_table","content":"pub fn from_table(table: Table) -&gt; Query\n\nSynonyms:\normlette/query/query.from_table\normlette/query/query from_table","url":"ormlette/query/query.html#from_table"},{"doc":"ormlette/query/query","title":"greater_than","content":"pub fn greater_than(\n  query: Query,\n  column: String,\n  value: a,\n) -&gt; Query\n\nSynonyms:\normlette/query/query.greater_than\normlette/query/query greater_than","url":"ormlette/query/query.html#greater_than"},{"doc":"ormlette/query/query","title":"inner_join","content":"pub fn inner_join(query: Query, target_table: Table) -&gt; Query\n\nSynonyms:\normlette/query/query.inner_join\normlette/query/query inner_join","url":"ormlette/query/query.html#inner_join"},{"doc":"ormlette/query/query","title":"join","content":"pub fn join(\n  query: Query,\n  target_table: Table,\n  join_type: fn(JoinTarget, Where, String) -&gt; Join,\n) -&gt; Query\n\nSynonyms:\normlette/query/query.join\normlette/query/query join","url":"ormlette/query/query.html#join"},{"doc":"ormlette/query/query","title":"left_join","content":"pub fn left_join(query: Query, target_table: Table) -&gt; Query\n\nSynonyms:\normlette/query/query.left_join\normlette/query/query left_join","url":"ormlette/query/query.html#left_join"},{"doc":"ormlette/query/query","title":"less_than","content":"pub fn less_than(query: Query, column: String, value: a) -&gt; Query\n\nSynonyms:\normlette/query/query.less_than\normlette/query/query less_than","url":"ormlette/query/query.html#less_than"},{"doc":"ormlette/query/query","title":"order_by","content":"pub fn order_by(\n  query: Query,\n  column: String,\n  direction: Direction,\n) -&gt; Query\n\nSynonyms:\normlette/query/query.order_by\normlette/query/query order_by","url":"ormlette/query/query.html#order_by"},{"doc":"ormlette/query/query","title":"select","content":"pub fn select(query: Query, columns: List(String)) -&gt; Query\n\nSynonyms:\normlette/query/query.select\normlette/query/query select","url":"ormlette/query/query.html#select"},{"doc":"ormlette/query/query","title":"sql","content":"pub fn sql(query: Query) -&gt; #(String, List(Param))\n\nSynonyms:\normlette/query/query.sql\normlette/query/query sql","url":"ormlette/query/query.html#sql"},{"doc":"ormlette/query/query","title":"ormlette/query/query","content":" The main Query module that provides a high-level API for building SQL queries\n It is built on top of the Cake library to provide a more ergonomic interface, but can easily be exported\n to standard cake-select queries.","url":"ormlette/query/query.html"},{"doc":"ormlette/query/run","title":"param_to_value","content":"pub fn param_to_value(v: Param) -&gt; Value\n\nSynonyms:\normlette/query/run.param_to_value\normlette/query/run param_to_value","url":"ormlette/query/run.html#param_to_value"},{"doc":"ormlette/query/run","title":"run","content":"pub fn run(\n  sql: #(String, List(Param)),\n  db: Connection,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Returned(a), QueryError)\n\nSynonyms:\normlette/query/run.run\normlette/query/run run","url":"ormlette/query/run.html#run"},{"doc":"ormlette/query/run","title":"ormlette/query/run","content":" This is used to help run the output of `src/ormlette/query/query.sql()` output","url":"ormlette/query/run.html"},{"doc":"ormlette/templates/utils/listy","title":"index","content":"pub fn index(list: List(a), index: Int) -&gt; a\n\nSynonyms:\normlette/templates/utils/listy.index\normlette/templates/utils/listy index","url":"ormlette/templates/utils/listy.html#index"},{"doc":"ormlette/templates/utils/listy","title":"ormlette/templates/utils/listy","content":"","url":"ormlette/templates/utils/listy.html"},{"doc":"ormlette/templates/utils/combine","title":"ColumnInfo","content":"pub type ColumnInfo {\n  ColumnInfo(name: String, reference_table: String)\n}\n\nColumnInfo(name: String, reference_table: String)\n\n\nSynonyms:\normlette/templates/utils/combine.ColumnInfo\normlette/templates/utils/combine ColumnInfo","url":"ormlette/templates/utils/combine.html#ColumnInfo"},{"doc":"ormlette/templates/utils/combine","title":"combine","content":"pub fn combine(table: Table) -&gt; List(Table)\n\nSynonyms:\normlette/templates/utils/combine.combine\normlette/templates/utils/combine combine","url":"ormlette/templates/utils/combine.html#combine"},{"doc":"ormlette/templates/utils/combine","title":"combine_tables","content":"pub fn combine_tables(t1: Table, t2: Table) -&gt; Table\n\nSynonyms:\normlette/templates/utils/combine.combine_tables\normlette/templates/utils/combine combine_tables","url":"ormlette/templates/utils/combine.html#combine_tables"},{"doc":"ormlette/templates/utils/combine","title":"get_column_info","content":"pub fn get_column_info(table: Table) -&gt; List(ColumnInfo)\n\nSynonyms:\normlette/templates/utils/combine.get_column_info\normlette/templates/utils/combine get_column_info","url":"ormlette/templates/utils/combine.html#get_column_info"},{"doc":"ormlette/templates/utils/combine","title":"table_has_fk","content":"pub fn table_has_fk(table: Table) -&gt; Option(List(Column))\n\nSynonyms:\normlette/templates/utils/combine.table_has_fk\normlette/templates/utils/combine table_has_fk","url":"ormlette/templates/utils/combine.html#table_has_fk"},{"doc":"ormlette/templates/utils/combine","title":"ormlette/templates/utils/combine","content":"","url":"ormlette/templates/utils/combine.html"},{"doc":"ormlette/templates/decoder_type","title":"render","content":"pub fn render(tables tables: List(Table)) -&gt; String\n\nSynonyms:\normlette/templates/decoder_type.render\normlette/templates/decoder_type render","url":"ormlette/templates/decoder_type.html#render"},{"doc":"ormlette/templates/decoder_type","title":"render_builder","content":"pub fn render_builder(\n  tables tables: List(Table),\n) -&gt; StringBuilder\n\nSynonyms:\normlette/templates/decoder_type.render_builder\normlette/templates/decoder_type render_builder","url":"ormlette/templates/decoder_type.html#render_builder"},{"doc":"ormlette/templates/decoder_type","title":"ormlette/templates/decoder_type","content":"","url":"ormlette/templates/decoder_type.html"},{"doc":"ormlette/templates/tables","title":"render","content":"pub fn render(tables tables: List(Table)) -&gt; String\n\nSynonyms:\normlette/templates/tables.render\normlette/templates/tables render","url":"ormlette/templates/tables.html#render"},{"doc":"ormlette/templates/tables","title":"render_builder","content":"pub fn render_builder(\n  tables tables: List(Table),\n) -&gt; StringBuilder\n\nSynonyms:\normlette/templates/tables.render_builder\normlette/templates/tables render_builder","url":"ormlette/templates/tables.html#render_builder"},{"doc":"ormlette/templates/tables","title":"ormlette/templates/tables","content":"","url":"ormlette/templates/tables.html"}]);